@@ -37,15 +37,10 @@
 
 #include <gio/gio.h>
 #include <glib/gstdio.h>
+#include <httpseverywhere.h>
 #include <libsoup/soup.h>
 #include <string.h>
 
-/* https://bugzilla.gnome.org/show_bug.cgi?id=772661 */
-#pragma GCC diagnostic push
-#pragma GCC diagnostic ignored "-Wredundant-decls"
-#include <httpseverywhere.h>
-#pragma GCC diagnostic pop
-
 #define DEFAULT_FILTER_URL "https://easylist-downloads.adblockplus.org/easylist.txt"
 #define FILTERS_LIST_FILENAME "filters.list"
 #define SIGNATURE_SIZE 8
@@ -79,6 +74,8 @@ struct _EphyUriTester {
 
   HTTPSEverywhereContext *https_everywhere_context;
   GList *deferred_requests;
+
+  GList *dbus_peers;
 };
 
 enum {
@@ -880,6 +877,43 @@ ephy_uri_tester_rewrite_uri (EphyUriTester *tester,
   return result;
 }
 
+typedef struct {
+  EphyUriTester *tester;
+  GDBusConnection *connection;
+  guint registration_id;
+} DBusPeerInfo;
+
+static DBusPeerInfo *
+dbus_peer_info_new (EphyUriTester   *tester,
+                    GDBusConnection *connection,
+                    guint            registration_id)
+{
+  DBusPeerInfo *peer = g_slice_new (DBusPeerInfo);
+  peer->tester = tester;
+  peer->connection = g_object_ref (connection);
+  peer->registration_id = registration_id;
+  return peer;
+}
+
+static void
+dbus_peer_info_free (DBusPeerInfo *peer)
+{
+  g_signal_handlers_disconnect_by_data (peer->connection, peer);
+
+  g_object_unref (peer->connection);
+  g_slice_free (DBusPeerInfo, peer);
+}
+
+static void
+dbus_connection_closed_cb (GDBusConnection *connection,
+                           gboolean         remote_peer_vanished,
+                           GError          *error,
+                           DBusPeerInfo    *peer)
+{
+  peer->tester->dbus_peers = g_list_remove (peer->tester->dbus_peers, peer);
+  dbus_peer_info_free (peer);
+}
+
 static void
 ephy_uri_tester_return_response (EphyUriTester         *tester,
                                  const char            *request_uri,
@@ -943,10 +977,11 @@ static const GDBusInterfaceVTable interface_vtable = {
 };
 
 void
-ephy_uri_tester_register_dbus_object (EphyUriTester   *tester,
-                                      GDBusConnection *connection)
+ephy_uri_tester_handle_new_dbus_connection (EphyUriTester   *tester,
+                                            GDBusConnection *connection)
 {
   static GDBusNodeInfo *introspection_data = NULL;
+  DBusPeerInfo *peer;
   guint registration_id;
   GError *error = NULL;
 
@@ -966,6 +1001,12 @@ ephy_uri_tester_register_dbus_object (EphyUriTester   *tester,
     g_error_free (error);
     return;
   }
+
+  peer = dbus_peer_info_new (tester, connection, registration_id);
+  tester->dbus_peers = g_list_append (tester->dbus_peers, peer);
+
+  g_signal_connect (connection, "closed",
+                    G_CALLBACK (dbus_connection_closed_cb), peer);
 }
 
 static void
@@ -988,6 +1029,7 @@ https_everywhere_update_cb (HTTPSEverywhereUpdater *updater,
 
   if (error) {
     if (!g_error_matches (error, G_IO_ERROR, G_IO_ERROR_CANCELLED) &&
+        !g_error_matches (error, HTTPS_EVERYWHERE_UPDATE_ERROR, HTTPS_EVERYWHERE_UPDATE_ERROR_IN_PROGRESS) &&
         !g_error_matches (error, HTTPS_EVERYWHERE_UPDATE_ERROR, HTTPS_EVERYWHERE_UPDATE_ERROR_NO_UPDATE_AVAILABLE))
       g_warning ("Failed to update HTTPS Everywhere rulesets: %s", error->message);
     g_error_free (error);
@@ -1042,6 +1084,8 @@ https_everywhere_context_init_cb (HTTPSEverywhereContext *context,
   g_list_free_full (tester->deferred_requests, (GDestroyNotify)deferred_request_free);
 
   ephy_uri_tester_update_https_everywhere_rulesets (tester);
+
+  g_object_unref (tester);
 }
 
 static void
@@ -1110,7 +1154,7 @@ ephy_uri_tester_constructed (GObject *object)
   https_everywhere_context_init (tester->https_everywhere_context,
                                  tester->cancellable,
                                  (GAsyncReadyCallback)https_everywhere_context_init_cb,
-                                 tester);
+                                 g_object_ref (tester));
 
   ephy_uri_tester_load_filters (tester);
   ephy_uri_tester_load_patterns (tester);
@@ -1137,6 +1181,13 @@ ephy_uri_tester_set_property (GObject      *object,
   }
 }
 
+static void
+unregister_dbus_object (DBusPeerInfo  *peer,
+                        EphyUriTester *tester)
+{
+  g_dbus_connection_unregister_object (peer->connection, peer->registration_id);
+}
+
 static void
 ephy_uri_tester_dispose (GObject *object)
 {
@@ -1144,7 +1195,16 @@ ephy_uri_tester_dispose (GObject *object)
 
   LOG ("EphyUriTester disposing %p", object);
 
-  g_clear_object (&tester->cancellable);
+  if (tester->cancellable) {
+    g_cancellable_cancel (tester->cancellable);
+    g_clear_object (&tester->cancellable);
+  }
+
+  if (tester->dbus_peers) {
+    g_list_foreach (tester->dbus_peers, (GFunc)unregister_dbus_object, tester);
+    g_list_free_full (tester->dbus_peers, (GDestroyNotify)dbus_peer_info_free);
+    tester->dbus_peers = NULL;
+  }
 
   G_OBJECT_CLASS (ephy_uri_tester_parent_class)->dispose (object);
 }
@@ -1209,5 +1269,21 @@ ephy_uri_tester_class_init (EphyUriTesterClass *klass)
 EphyUriTester *
 ephy_uri_tester_new (void)
 {
-  return g_object_new (EPHY_TYPE_URI_TESTER, "base-data-dir", ephy_dot_dir (), NULL);
+  EphyEmbedShell *shell;
+  EphyEmbedShellMode mode;
+  EphyUriTester *tester;
+  char *data_dir;
+
+  shell = ephy_embed_shell_get_default ();
+  mode = ephy_embed_shell_get_mode (shell);
+
+  /* The filters list is large, so we don't want to store a separate copy per
+   * web app, but users should otherwise be able to configure different filters
+   * per profile directory. */
+  data_dir = mode == EPHY_EMBED_SHELL_MODE_APPLICATION ? ephy_default_dot_dir ()
+                                                       : g_strdup (ephy_dot_dir ());
+
+  tester = g_object_new (EPHY_TYPE_URI_TESTER, "base-data-dir", data_dir, NULL);
+  g_free (data_dir);
+  return tester;
 }
