@@ -1,18 +1,21 @@
+/* -*- Mode: C; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
 /*
  *  Copyright © 2012 Igalia S.L.
  *
- *  This program is free software; you can redistribute it and/or modify
- *  it under the terms of the GNU General Public License as published by
- *  the Free Software Foundation; either version 2, or (at your option)
- *  any later version.
+ *  This file is part of Epiphany.
  *
- *  This program is distributed in the hope that it will be useful,
+ *  Epiphany is free software: you can redistribute it and/or modify
+ *  it under the terms of the GNU General Public License as published by
+ *  the Free Software Foundation, either version 3 of the License, or
+ *  (at your option) any later version.
+ *
+ *  Epiphany is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  *  GNU General Public License for more details.
  *
  *  You should have received a copy of the GNU General Public License
- *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
+ *  along with Epiphany.  If not, see <http://www.gnu.org/licenses/>.
  */
 
 #include "config.h"
@@ -301,7 +304,7 @@ ephy_certificate_dialog_init (EphyCertificateDialog *dialog)
   gtk_label_set_selectable (GTK_LABEL (dialog->title), TRUE);
   gtk_widget_set_halign (dialog->title, GTK_ALIGN_START);
   gtk_widget_set_valign (dialog->title, GTK_ALIGN_CENTER);
-  gtk_misc_set_alignment (GTK_MISC (dialog->title), 0.0, 0.5);
+  gtk_label_set_xalign (GTK_LABEL (dialog->title), 0.0);
   gtk_grid_attach_next_to (GTK_GRID (grid), dialog->title,
                            dialog->icon, GTK_POS_RIGHT,
                            1, 1);
@@ -312,7 +315,8 @@ ephy_certificate_dialog_init (EphyCertificateDialog *dialog)
   gtk_label_set_selectable (GTK_LABEL (dialog->text), TRUE);
   gtk_widget_set_halign (dialog->text, GTK_ALIGN_START);
   gtk_widget_set_valign (dialog->text, GTK_ALIGN_START);
-  gtk_misc_set_alignment (GTK_MISC (dialog->text), 0.0, 0.0);
+  gtk_label_set_xalign (GTK_LABEL (dialog->text), 0.0);
+  gtk_label_set_yalign (GTK_LABEL (dialog->text), 0.0);
   gtk_grid_attach_next_to (GTK_GRID (grid), dialog->text,
                            dialog->title, GTK_POS_BOTTOM,
                            1, 1);
