@@ -38,14 +38,13 @@ struct _EphyBookmarksManager {
   gchar      *gvdb_filename;
 };
 
-static void list_model_iface_init     (GListModelInterface *iface);
-
-G_DEFINE_TYPE_EXTENDED (EphyBookmarksManager, ephy_bookmarks_manager, G_TYPE_OBJECT, 0,
-                        G_IMPLEMENT_INTERFACE (G_TYPE_LIST_MODEL, list_model_iface_init))
+G_DEFINE_TYPE (EphyBookmarksManager, ephy_bookmarks_manager, G_TYPE_OBJECT)
 
 enum {
   BOOKMARK_ADDED,
   BOOKMARK_REMOVED,
+  BOOKMARK_TITLE_CHANGED,
+  BOOKMARK_URL_CHANGED,
   TAG_CREATED,
   TAG_DELETED,
   LAST_SIGNAL
@@ -148,7 +147,7 @@ ephy_bookmarks_manager_class_init (EphyBookmarksManagerClass *klass)
   object_class->finalize = ephy_bookmarks_manager_finalize;
 
   signals[BOOKMARK_ADDED] =
-      g_signal_new ("bookmark-added",
+    g_signal_new ("bookmark-added",
                   EPHY_TYPE_BOOKMARKS_MANAGER,
                   G_SIGNAL_RUN_LAST,
                   0,
@@ -165,6 +164,24 @@ ephy_bookmarks_manager_class_init (EphyBookmarksManagerClass *klass)
                   G_TYPE_NONE, 1,
                   EPHY_TYPE_BOOKMARK);
 
+  signals[BOOKMARK_TITLE_CHANGED] =
+    g_signal_new ("bookmark-title-changed",
+                  EPHY_TYPE_BOOKMARKS_MANAGER,
+                  G_SIGNAL_RUN_LAST,
+                  0,
+                  NULL, NULL, NULL,
+                  G_TYPE_NONE, 1,
+                  EPHY_TYPE_BOOKMARK);
+
+  signals[BOOKMARK_URL_CHANGED] =
+    g_signal_new ("bookmark-url-changed",
+                  EPHY_TYPE_BOOKMARKS_MANAGER,
+                  G_SIGNAL_RUN_LAST,
+                  0,
+                  NULL, NULL, NULL,
+                  G_TYPE_NONE, 1,
+                  EPHY_TYPE_BOOKMARK);
+
   signals[TAG_CREATED] =
     g_signal_new ("tag-created",
                   EPHY_TYPE_BOOKMARKS_MANAGER,
@@ -206,38 +223,20 @@ ephy_bookmarks_manager_init (EphyBookmarksManager *self)
   ephy_bookmarks_manager_load_from_file (self);
 }
 
-static GType
-ephy_bookmarks_manager_list_model_get_item_type (GListModel *model)
+static void
+bookmark_title_changed_cb (EphyBookmark         *bookmark,
+                           GParamSpec           *pspec,
+                           EphyBookmarksManager *self)
 {
-  return EPHY_TYPE_BOOKMARK;
-}
-
-static guint
-ephy_bookmarks_manager_list_model_get_n_items (GListModel *model)
-{
-  EphyBookmarksManager *self = EPHY_BOOKMARKS_MANAGER (model);
-
-  return g_sequence_get_length (self->bookmarks);
-}
-
-static gpointer
-ephy_bookmarks_manager_list_model_get_item (GListModel *model,
-                                            guint       position)
-{
-  EphyBookmarksManager *self = EPHY_BOOKMARKS_MANAGER (model);
-  GSequenceIter *iter;
-
-  iter = g_sequence_get_iter_at_pos (self->bookmarks, position);
-
-  return g_object_ref (g_sequence_get (iter));
+  g_signal_emit (self, signals[BOOKMARK_TITLE_CHANGED], 0, bookmark);
 }
 
 static void
-list_model_iface_init (GListModelInterface *iface)
+bookmark_url_changed_cb (EphyBookmark         *bookmark,
+                         GParamSpec           *pspec,
+                         EphyBookmarksManager *self)
 {
-  iface->get_item_type = ephy_bookmarks_manager_list_model_get_item_type;
-  iface->get_n_items = ephy_bookmarks_manager_list_model_get_n_items;
-  iface->get_item = ephy_bookmarks_manager_list_model_get_item;
+  g_signal_emit (self, signals[BOOKMARK_URL_CHANGED], 0, bookmark);
 }
 
 void
@@ -246,7 +245,6 @@ ephy_bookmarks_manager_add_bookmark (EphyBookmarksManager *self,
 {
   GSequenceIter *iter;
   GSequenceIter *prev_iter;
-  int position;
 
   g_return_if_fail (EPHY_IS_BOOKMARKS_MANAGER (self));
   g_return_if_fail (EPHY_IS_BOOKMARK (bookmark));
@@ -259,18 +257,17 @@ ephy_bookmarks_manager_add_bookmark (EphyBookmarksManager *self,
   prev_iter = g_sequence_iter_prev (iter);
   if (g_sequence_iter_is_end (prev_iter)
       || ephy_bookmark_get_time_added (g_sequence_get (prev_iter)) != ephy_bookmark_get_time_added (bookmark)) {
+    g_sequence_insert_before (iter, bookmark);
+    g_signal_emit (self, signals[BOOKMARK_ADDED], 0, bookmark);
 
-      g_sequence_insert_before (iter, bookmark);
+    ephy_bookmarks_manager_save_to_file_async (self, NULL,
+                                               (GAsyncReadyCallback)ephy_bookmarks_manager_save_to_file_warn_on_error_cb,
+                                               NULL);
 
-      /* Update list */
-      position = g_sequence_iter_get_position (iter);
-      g_list_model_items_changed (G_LIST_MODEL (self), position - 1, 0, 1);
-
-      g_signal_emit (self, signals[BOOKMARK_ADDED], 0, bookmark);
-
-      ephy_bookmarks_manager_save_to_file_async (self, NULL,
-                                                 (GAsyncReadyCallback)ephy_bookmarks_manager_save_to_file_warn_on_error_cb,
-                                                 NULL);
+    g_signal_connect_object (bookmark, "notify::title",
+                             G_CALLBACK (bookmark_title_changed_cb), self, 0);
+    g_signal_connect_object (bookmark, "notify::url",
+                             G_CALLBACK (bookmark_url_changed_cb), self, 0);
   }
 }
 
@@ -309,7 +306,6 @@ ephy_bookmarks_manager_remove_bookmark (EphyBookmarksManager *self,
                                         EphyBookmark         *bookmark)
 {
   GSequenceIter *iter;
-  int position;
 
   g_return_if_fail (EPHY_IS_BOOKMARKS_MANAGER (self));
   g_return_if_fail (EPHY_IS_BOOKMARK (bookmark));
@@ -323,14 +319,14 @@ ephy_bookmarks_manager_remove_bookmark (EphyBookmarksManager *self,
   }
 
   g_signal_emit (self, signals[BOOKMARK_REMOVED], 0, bookmark);
-
-  position = g_sequence_iter_get_position (iter);
   g_sequence_remove (iter);
-  g_list_model_items_changed (G_LIST_MODEL (self), position, 1, 0);
 
   ephy_bookmarks_manager_save_to_file_async (self, NULL,
                                              (GAsyncReadyCallback)ephy_bookmarks_manager_save_to_file_warn_on_error_cb,
                                              NULL);
+
+  g_signal_handlers_disconnect_by_func (bookmark, bookmark_title_changed_cb, self);
+  g_signal_handlers_disconnect_by_func (bookmark, bookmark_url_changed_cb, self);
 }
 
 EphyBookmark *
@@ -465,7 +461,7 @@ ephy_bookmarks_manager_get_bookmarks_with_tag (EphyBookmarksManager *self,
 
       if (g_sequence_is_empty (ephy_bookmark_get_tags (bookmark))) {
         g_sequence_insert_sorted (bookmarks,
-                                  bookmark,
+                                  g_object_ref (bookmark),
                                   (GCompareDataFunc)ephy_bookmark_bookmarks_sort_func,
                                   NULL);
       }
@@ -478,7 +474,7 @@ ephy_bookmarks_manager_get_bookmarks_with_tag (EphyBookmarksManager *self,
 
       if (ephy_bookmark_has_tag (bookmark, tag))
         g_sequence_insert_sorted (bookmarks,
-                                  bookmark,
+                                  g_object_ref (bookmark),
                                   (GCompareDataFunc)ephy_bookmark_bookmarks_sort_func,
                                   NULL);
     }
@@ -487,6 +483,39 @@ ephy_bookmarks_manager_get_bookmarks_with_tag (EphyBookmarksManager *self,
   return bookmarks;
 }
 
+static int
+compare_smart_bookmarks (EphyBookmark *a,
+                         EphyBookmark *b)
+{
+  return g_utf8_collate (ephy_bookmark_get_title (a), ephy_bookmark_get_title (b));
+}
+
+GSequence *
+ephy_bookmarks_manager_get_smart_bookmarks (EphyBookmarksManager *self)
+{
+  GSequence *bookmarks;
+  GSequenceIter *iter;
+
+  g_return_val_if_fail (EPHY_IS_BOOKMARKS_MANAGER (self), NULL);
+
+  bookmarks = g_sequence_new (g_object_unref);
+
+  for (iter = g_sequence_get_begin_iter (self->bookmarks);
+       !g_sequence_iter_is_end (iter);
+       iter = g_sequence_iter_next (iter)) {
+    EphyBookmark *bookmark = g_sequence_get (iter);
+
+    if (ephy_bookmark_is_smart (bookmark)) {
+      g_sequence_insert_sorted (bookmarks,
+                                g_object_ref (bookmark),
+                                (GCompareDataFunc)compare_smart_bookmarks,
+                                NULL);
+    }
+  }
+
+  return bookmarks;
+}
+
 GSequence *
 ephy_bookmarks_manager_get_tags (EphyBookmarksManager *self)
 {
