@@ -33,7 +33,6 @@
 #include "ephy-history-window.h"
 #include "ephy-lockdown.h"
 #include "ephy-prefs.h"
-#include "ephy-private.h"
 #include "ephy-session.h"
 #include "ephy-settings.h"
 #include "ephy-title-box.h"
@@ -905,6 +904,13 @@ ephy_shell_close_all_windows (EphyShell *shell)
   return retval;
 }
 
+void
+ephy_shell_try_quit (EphyShell *shell)
+{
+  if (ephy_shell_close_all_windows (shell))
+    g_application_quit (G_APPLICATION (shell));
+}
+
 typedef struct {
   EphyShell *shell;
   EphySession *session;
@@ -986,7 +992,7 @@ ephy_shell_open_uris_idle (OpenURIsData *data)
     page_flags |= EPHY_NEW_TAB_APPEND_AFTER;
   else if (data->reuse_empty_tab) {
     embed = ephy_embed_container_get_active_child (EPHY_EMBED_CONTAINER (data->window));
-    /* Only load a new page in this embed if it was showing or loading the homepage about:overview */
+    /* Only load a new page in this embed if it was showing or loading the homepage */
     if (ephy_web_view_get_visit_type (ephy_embed_get_web_view (embed)) == EPHY_PAGE_VISIT_HOMEPAGE)
       reusing_empty_tab = TRUE;
   }
@@ -1054,4 +1060,3 @@ ephy_shell_open_uris (EphyShell       *shell,
 
   shell->open_uris_idle_ids = g_slist_prepend (shell->open_uris_idle_ids, GUINT_TO_POINTER (id));
 }
-
